@@ -1,15 +1,13 @@
-'''Enhanced threading.Thread which can deliver a return value and propagate
-exceptions from the called thread to the calling thread.
+'''Enhanced Thread with support for return values and exception propagation.'''
 
-Copyright (C) 2007 Canonical Ltd.
-Author: Martin Pitt <martin.pitt@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
-'''
+# Copyright (C) 2007 Canonical Ltd.
+# Author: Martin Pitt <martin.pitt@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
 
 import threading, sys
 
@@ -78,7 +76,7 @@ if __name__ == '__main__':
 
         return x / y
 
-    class _REThreadTest(unittest.TestCase):
+    class _T(unittest.TestCase):
         def test_return_value(self):
             '''return value works properly.'''
 
