@@ -1,16 +1,15 @@
 # coding=utf-8
 
-'''Simple in-memory CrashDatabase implementation, mainly useful for testing.
+'''Simple in-memory CrashDatabase implementation, mainly useful for testing.'''
 
-Copyright (C) 2007 Canonical Ltd.
-Author: Martin Pitt <martin.pitt@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
-'''
+# Copyright (C) 2007 - 2009 Canonical Ltd.
+# Author: Martin Pitt <martin.pitt@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
 
 import copy, time, os, unittest
 import apport.crashdb
@@ -66,13 +65,52 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
 
         return self.reports[id]['report']
 
-    def update(self, id, report, comment = ''):
-        '''Update the given report ID with the retraced results from the report
-        (Stacktrace, ThreadStacktrace, StacktraceTop; also Disassembly if
-        desired) and an optional comment.'''
+    def get_affected_packages(self, id):
+        '''Return list of affected source packages for given ID.'''
 
-        self.reports[id]['report'] = report
-        self.reports[id]['comment'] = comment
+        return [self.reports[id]['report']['SourcePackage']]
+
+    def is_reporter(self, id):
+        '''Check whether the user is the reporter of given ID.'''
+
+        return True
+
+    def can_update(self, id):
+        '''Check whether the user is eligible to update a report.
+
+        A user should add additional information to an existing ID if (s)he is
+        the reporter or subscribed, the bug is open, not a duplicate, etc. The
+        exact policy and checks should be done according to  the particular
+        implementation.
+        '''
+        return self.is_reporter(id)
+
+    def update(self, id, report, comment, change_description=False,
+            attachment_comment=None, key_filter=None):
+        '''Update the given report ID with all data from report.
+
+        This creates a text comment with the "short" data (see
+        ProblemReport.write_mime()), and creates attachments for all the
+        bulk/binary data. 
+        
+        If change_description is True, and the crash db implementation supports
+        it, the short data will be put into the description instead (like in a
+        new bug).
+
+        comment will be added to the "short" data. If attachment_comment is
+        given, it will be added to the attachment uploads.
+
+        If key_filter is a list or set, then only those keys will be added.
+        '''
+        r = self.reports[id]
+        r['comment'] = comment
+
+        if key_filter:
+            for f in key_filter:
+                if f in report:
+                    r['report'][f] = report[f]
+        else:
+            r['report'].update(report)
 
     def get_distro_release(self, id):
         '''Get 'DistroRelease: <release>' from the given report ID and return
@@ -261,7 +299,7 @@ ZeroDivisionError: integer division or modulo by zero'''
 # Unit test (this also tests the dup detection API from apport/crashdb.py)
 #
 
-class _MemoryCrashDBTest(unittest.TestCase):
+class _T(unittest.TestCase):
     def setUp(self):
         self.crashes = CrashDatabase(None, None, {'dummy_data': '1'})
 
@@ -306,17 +344,59 @@ class _MemoryCrashDBTest(unittest.TestCase):
 
         self.assertRaises(IndexError, self.crashes.download, 5)
 
+    def test_get_affected_packages(self):
+        self.assertEqual(self.crashes.get_affected_packages(0), ['foo'])
+        self.assertEqual(self.crashes.get_affected_packages(1), ['foo'])
+        self.assertEqual(self.crashes.get_affected_packages(2), ['bar'])
+        self.assertEqual(self.crashes.get_affected_packages(3), ['pygoo'])
+
     def test_update(self):
         '''update()'''
 
         r = apport.Report()
         r['Package'] = 'new'
+        r['FooBar'] = 'Bogus'
+        r['StacktraceTop'] = 'Fresh!'
 
         self.crashes.update(1, r, 'muhaha')
-        self.assertEqual(self.crashes.download(1)['Package'], 'new')
         self.assertEqual(self.crashes.reports[1]['comment'], 'muhaha')
+        self.assertEqual(self.crashes.download(1)['Package'], 'new')
+        self.assertEqual(self.crashes.download(1)['StacktraceTop'], 'Fresh!')
+        self.assertEqual(self.crashes.download(1)['FooBar'], 'Bogus')
 
-        self.assertRaises(IndexError, self.crashes.update, 5, None)
+        self.assertRaises(IndexError, self.crashes.update, 5, None, '')
+
+    def test_update_filter(self):
+        '''update() with key_filter'''
+
+        r = apport.Report()
+        r['Package'] = 'new'
+        r['FooBar'] = 'Bogus'
+        r['StacktraceTop'] = 'Fresh!'
+
+        self.crashes.update(1, r, 'muhaha', key_filter=['FooBar', 'StacktraceTop'])
+        self.assertEqual(self.crashes.reports[1]['comment'], 'muhaha')
+        self.assertEqual(self.crashes.download(1)['Package'], 'libfoo1 1.2-4')
+        self.assertEqual(self.crashes.download(1)['StacktraceTop'], 'Fresh!')
+        self.assertEqual(self.crashes.download(1)['FooBar'], 'Bogus')
+
+        self.assertRaises(IndexError, self.crashes.update, 5, None, '')
+
+    def test_update_traces(self):
+        '''update_traces()'''
+
+        r = apport.Report()
+        r['Package'] = 'new'
+        r['FooBar'] = 'Bogus'
+        r['StacktraceTop'] = 'Fresh!'
+
+        self.crashes.update_traces(1, r, 'muhaha')
+        self.assertEqual(self.crashes.reports[1]['comment'], 'muhaha')
+        self.assertEqual(self.crashes.download(1)['Package'], 'libfoo1 1.2-4')
+        self.assertEqual(self.crashes.download(1)['StacktraceTop'], 'Fresh!')
+        self.failIf(self.crashes.download(1).has_key('FooBar'))
+
+        self.assertRaises(IndexError, self.crashes.update_traces, 5, None)
 
     def test_get_distro_release(self):
         '''get_distro_release()'''
