@@ -1,14 +1,13 @@
-'''Functions to manage apport problem report files.
+'''Functions to manage apport problem report files.'''
 
-Copyright (C) 2006 Canonical Ltd.
-Author: Martin Pitt <martin.pitt@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
-'''
+# Copyright (C) 2006 - 2009 Canonical Ltd.
+# Author: Martin Pitt <martin.pitt@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
 
 import os, glob, subprocess, os.path, ConfigParser
 from problem_report import ProblemReport
@@ -255,7 +254,7 @@ get_config.config = None
 import unittest, tempfile, os, shutil, sys, time
 from cStringIO import StringIO
 
-class _ApportUtilsTest(unittest.TestCase):
+class _T(unittest.TestCase):
     def setUp(self):
         global report_dir
         global _config_file
