@@ -1,15 +1,14 @@
-'''Convenience functions for use in package hooks.
+'''Convenience functions for use in package hooks.'''
 
-Copyright (C) 2008-2009 Canonical Ltd.
-Author: Matt Zimmerman <mdz@canonical.com>
-Contributor: Brian Murray <brian@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
-'''
+# Copyright (C) 2008 - 2009 Canonical Ltd.
+# Author: Matt Zimmerman <mdz@canonical.com>
+# Contributor: Brian Murray <brian@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
 
 import subprocess
 import hashlib
@@ -69,6 +68,25 @@ def attach_dmesg(report):
     report['BootDmesg'] = open('/var/log/dmesg').read()
     report['CurrentDmesg'] = command_output(['sh', '-c', 'dmesg | comm -13 --nocheck-order /var/log/dmesg -'])
 
+def attach_dmi(report):
+    dmi_dir = '/sys/class/dmi/id'
+    if os.path.isdir(dmi_dir):
+        for f in os.listdir(dmi_dir):
+            p = '%s/%s' % (dmi_dir, f)
+            st = os.stat(p)
+            # ignore the root-only ones, since they have serial numbers
+            if not stat.S_ISREG(st.st_mode) or (st.st_mode & 4 == 0):
+                continue
+            if f in ('subsystem', 'uevent'):
+                continue
+
+            try:
+                value = open(p).read().strip()
+            except (OSError, IOError):
+                continue
+            if value:
+                report['dmi.' + f.replace('_', '.')] = value
+
 def attach_hardware(report):
     '''Attach a standard set of hardware-related data to the report, including:
 
@@ -96,23 +114,15 @@ def attach_hardware(report):
     report['Lsusb'] = command_output(['lsusb'])
     report['UdevDb'] = command_output(['udevadm', 'info', '--export-db'])
 
-    dmi_dir = '/sys/class/dmi/id'
-    if os.path.isdir(dmi_dir):
-        for f in os.listdir(dmi_dir):
-            p = '%s/%s' % (dmi_dir, f)
-            st = os.stat(p)
-            # ignore the root-only ones, since they have serial numbers
-            if not stat.S_ISREG(st.st_mode) or (st.st_mode & 4 == 0):
-                continue
-            if f in ('subsystem', 'uevent'):
-                continue
+    # anonymize partition labels
+    report['UdevLog'] = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', report['UdevLog'])
+    report['UdevLog'] = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', report['UdevLog'])
+    report['UdevLog'] = re.sub('by-label/(.*)', 'by-label/<hidden>', report['UdevLog'])
+    report['UdevDb'] = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', report['UdevDb'])
+    report['UdevDb'] = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', report['UdevDb'])
+    report['UdevDb'] = re.sub('by-label/(.*)', 'by-label/<hidden>', report['UdevDb'])
 
-            try:
-                value = open(p).read().strip()
-            except (OSError, IOError):
-                continue
-            if value:
-                report['dmi.' + f.replace('_', '.')] = value
+    attach_dmi(report)
 
     # Use the hardware information to create a machine type.
     if 'dmi.sys.vendor' in report and 'dmi.product.name' in report:
@@ -140,6 +150,8 @@ def attach_alsa(report):
     attach_file_if_exists(report, os.path.expanduser('~/.asoundrc.asoundconf'),
                           'UserAsoundrcAsoundconf')
     attach_file_if_exists(report, '/etc/asound.conf')
+    attach_file_if_exists(report, '/proc/asound/version', 'AlsaVersion')
+    attach_file(report, '/proc/cpuinfo', 'ProcCpuinfo')
 
     report['AlsaDevices'] = command_output(['ls','-l','/dev/snd/'])
     report['AplayDevices'] = command_output(['aplay','-l'])
@@ -176,6 +188,7 @@ def attach_alsa(report):
             + glob.glob('/dev/snd/*')
             + glob.glob('/dev/seq*') )
 
+    attach_dmi(report)
     attach_dmesg(report)
 
     # This seems redundant with the amixer info, do we need it?
@@ -253,6 +266,22 @@ def recent_syslog(pattern):
             lines += line
     return lines
 
+def xsession_errors(pattern):
+    '''Extract messages from ~/.xsession-errors which match a regex.
+        
+    pattern should be a "re" object.
+    '''
+
+    path = os.path.expanduser('~/.xsession-errors')
+    if not os.path.exists(path):
+        return None
+
+    lines = ''
+    for line in open(path):
+        if pattern.search(line):
+            lines += line
+    return lines
+
 PCI_MASS_STORAGE = 0x01
 PCI_NETWORK = 0x02
 PCI_DISPLAY = 0x03
@@ -370,7 +399,7 @@ def attach_wifi(report):
     '''Attach wireless (WiFi) network information to report.'''
 
     report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel):'))
-    report['IwConfig'] = command_output(['iwconfig'])
+    report['IwConfig'] = re.sub('Encryption key:(.*)', 'Encryption key: <hidden>', command_output(['iwconfig']))
     report['RfKill'] = command_output(['rfkill', 'list'])
     report['CRDA'] = command_output(['iw', 'reg', 'get'])
 
@@ -416,6 +445,8 @@ def package_versions(*packages):
     '''
     versions = ''
     for package_pattern in packages:
+        if not package_pattern:
+            continue
         for package in packaging.package_name_glob(package_pattern):
             try:
                 version = packaging.get_version(package)
@@ -427,6 +458,39 @@ def package_versions(*packages):
 
     return versions
 
+def shared_libraries(path):
+    '''Returns a list of strings containing the sonames of shared libraries
+    with which the specified binary is linked.'''
+
+    libs = set()
+
+    for line in command_output(['ldd', path]).split('\n'):
+        try:
+            lib, rest = line.split('=>', 1)
+        except ValueError:
+            continue
+
+        lib = lib.strip()
+        libs.add(lib)
+
+    return libs
+
+def links_with_shared_library(path, lib):
+    '''Returns True if the binary at path links with the library named lib.
+
+    path should be a fully qualified path (e.g. report['ExecutablePath'])
+    lib may be of the form 'lib<name>' or 'lib<name>.so.<version>'
+    '''
+
+    libs = shared_libraries(path)
+
+    if lib in libs: return True
+
+    for linked_lib in libs:
+        if linked_lib.startswith(lib + '.so.'): return True
+
+    return False
+
 def _get_module_license(module):
     '''Return the license for a given kernel module.'''
 
@@ -501,7 +565,7 @@ if __name__ == '__main__':
 
     import unittest, tempfile
 
-    class _ApportHookutilsTest(unittest.TestCase):
+    class _T(unittest.TestCase):
         def test_module_license_evaluation(self):
             '''module licenses can be validated correctly.'''
 
