@@ -1,15 +1,13 @@
-'''Class for an apport report with some useful methods to collect standard
-debug information.
+'''Representation of and data collection for a problem report.'''
 
-Copyright (C) 2006 Canonical Ltd.
-Author: Martin Pitt <martin.pitt@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
-'''
+# Copyright (C) 2006 - 2009 Canonical Ltd.
+# Author: Martin Pitt <martin.pitt@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
 
 import subprocess, tempfile, os.path, urllib, re, pwd, grp, os, sys
 import fnmatch, glob, atexit, traceback, errno
@@ -21,8 +19,9 @@ from problem_report import ProblemReport
 import fileutils
 from packaging_impl import impl as packaging
 
-_hook_dir = '/usr/share/apport/package-hooks/'
-_common_hook_dir = '/usr/share/apport/general-hooks/'
+_data_dir = os.environ.get('APPORT_DATA_DIR','/usr/share/apport')
+_hook_dir = '%s/package-hooks/' % (_data_dir)
+_common_hook_dir = '%s/general-hooks/' % (_data_dir)
 
 # path of the ignore file
 _ignore_file = '~/.apport-ignore.xml'
@@ -941,7 +940,7 @@ class Report(ProblemReport):
         '''Return list of obsolete packages in Package and Dependencies.'''
 
         obsolete = []
-        for l in (self['Package'] + '\n' + self.get('Dependencies', '')).splitlines():
+        for l in (self.get('Package', '') + '\n' + self.get('Dependencies', '')).splitlines():
             if not l:
                 continue
             pkg, ver = l.split()[:2]
@@ -1054,7 +1053,10 @@ class Report(ProblemReport):
             pass
 
         for k in self:
-            if k.startswith('Proc') or 'Stacktrace' in k or \
+            if (k.startswith('Proc') and \
+                not k in ['ProcCpuinfo','ProcMaps','ProcStatus', \
+                          'ProcInterrupts','ProcModules']) or \
+                'Stacktrace' in k or \
                 k in ['Traceback', 'PythonArgs']:
                 for old, new in replacements.iteritems():
                     if hasattr(self[k], 'isspace'):
@@ -1067,7 +1069,7 @@ class Report(ProblemReport):
 import unittest, shutil, signal, time
 from cStringIO import StringIO
 
-class _ApportReportTest(unittest.TestCase):
+class _T(unittest.TestCase):
     def test_add_package_info(self):
         '''add_package_info().'''
 
@@ -1471,7 +1473,7 @@ int main() {
 }
 ''')
         self._validate_gdb_fields(pr)
-        self.assertEqual(pr['Disassembly'], '0x0:\tCannot access memory at address 0x0', pr['Disassembly'])
+        self.assert_('Cannot access memory at address 0x0' in pr['Disassembly'], pr['Disassembly'])
         self.failIf ('AssertionMessage' in pr)
 
     def test_add_gdb_info_load(self):
@@ -1487,6 +1489,34 @@ int main() {
 
         self._validate_gdb_fields(pr)
 
+    def test_add_zz_parse_segv_details(self):
+        '''parse-segv produces sensible results'''
+        rep = tempfile.NamedTemporaryFile()
+        self._generate_sigsegv_report(rep)
+        rep.seek(0)
+
+        pr = Report()
+        pr.load(open(rep.name))
+        pr['Signal'] = '1'
+        pr.add_hooks_info('fake_ui')
+        self.assert_('SegvAnalysis' not in pr.keys())
+
+        pr = Report()
+        pr.load(open(rep.name))
+        pr.add_hooks_info('fake_ui')
+        self.assert_('Skipped: missing required field "Architecture"' in pr['SegvAnalysis'],
+                     pr['SegvAnalysis'])
+
+        pr.add_os_info()
+        pr.add_hooks_info('fake_ui')
+        self.assert_('Skipped: missing required field "ProcMaps"' in pr['SegvAnalysis'],
+                     pr['SegvAnalysis'])
+
+        pr.add_proc_info()
+        pr.add_hooks_info('fake_ui')
+        self.assert_('not located in a known VMA region' in pr['SegvAnalysis'],
+                     pr['SegvAnalysis'])
+
     def test_add_gdb_info_script(self):
         '''add_gdb_info() with a script.'''
 
@@ -2118,7 +2148,7 @@ baz()
         '''obsolete_packages().'''
 
         report = Report()
-        self.assertRaises(KeyError, report.obsolete_packages)
+        self.assertEqual(report.obsolete_packages(), [])
 
         # should work without Dependencies
         report['Package'] = 'bash 0'
@@ -2308,23 +2338,7 @@ ZeroDivisionError: integer division or modulo by zero'''
         r['ProblemType'] = 'KernelCrash'
         r['Stacktrace'] = '''
 crash 4.0-8.9
-Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Red Hat, Inc.
-Copyright (C) 2004, 2005, 2006  IBM Corporation
-Copyright (C) 1999-2006  Hewlett-Packard Co
-Copyright (C) 2005, 2006  Fujitsu Limited
-Copyright (C) 2006, 2007  VA Linux Systems Japan K.K.
-Copyright (C) 2005  NEC Corporation
-Copyright (C) 1999, 2002, 2007  Silicon Graphics, Inc.
-Copyright (C) 1999, 2000, 2001, 2002  Mission Critical Linux, Inc.
-This program is free software, covered by the GNU General Public License,
-and you are welcome to change it and/or distribute copies of it under
-certain conditions.  Enter "help copying" to see the conditions.
-This program has absolutely no warranty.  Enter "help warranty" for details.
- 
-NOTE: stdin: not a tty
-
 GNU gdb 6.1
-Copyright 2004 Free Software Foundation, Inc.
 GDB is free software, covered by the GNU General Public License, and you are
 welcome to change it and/or distribute copies of it under certain conditions.
 Type "show copying" to see the conditions.
