@@ -1,16 +1,17 @@
-'''An apport.PackageInfo class implementation for python-apt and dpkg, as found
-on Debian and derivatives such as Ubuntu.
+'''apport.PackageInfo class implementation for python-apt and dpkg.
 
-Copyright (C) 2007, 2009 Canonical Ltd.
-Author: Martin Pitt <martin.pitt@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
+This is used on Debian and derivatives such as Ubuntu.
 '''
 
+# Copyright (C) 2007 - 2009 Canonical Ltd.
+# Author: Martin Pitt <martin.pitt@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
+
 import subprocess, os, glob, stat, sys, tempfile, glob, re, shutil
 
 import warnings
@@ -46,7 +47,12 @@ class __AptDpkgPackageInfo(PackageInfo):
         Throw a ValueError if the package does not exist.
         '''
         if not self._apt_cache:
-            self._apt_cache = apt.Cache()
+            try:
+                # avoid spewage on stdout
+                self._apt_cache = apt.Cache(apt.progress.base.OpProgress())
+            except AttributeError:
+                # older python-apt versions do not yet have above argument
+                self._apt_cache = apt.Cache()
         return self._apt_cache
 
     def _apt_pkg(self, package):
@@ -368,7 +374,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         # urlretrieve does not return 404 in the headers so we use urlopen
         u = urllib.urlopen(url)
         if u.getcode() > 400:
-            raise IOError, 'urllib returned %s for %s' % (u.getcode(), url)
+            return ('', 'linux')
         while True:
             block = u.read(8*1024)
             if not block:
@@ -430,7 +436,12 @@ class __AptDpkgPackageInfo(PackageInfo):
         for l in (report['Package'] + '\n' + report.get('Dependencies', '')).splitlines():
             if not l.strip():
                 continue
-            (pkg, version) = l.split()[:2]
+            try:
+                (pkg, version) = l.split()[:2]
+            except ValueError:
+                print >> sys.stderr, 'WARNING: invalid Package/Dependencies line: ', l
+                # invalid line, ignore
+                continue
             dependency_versions[pkg] = version
             try:
                 if self.get_architecture(pkg) != 'all':
@@ -821,7 +832,7 @@ impl = __AptDpkgPackageInfo()
 if __name__ == '__main__':
     import unittest, gzip
 
-    class _AptDpkgPackageInfoTest(unittest.TestCase):
+    class _T(unittest.TestCase):
 
         def setUp(self):
             # save and restore configuration file
