@@ -3,17 +3,30 @@
 # of a segfault.  Currently very very simplistic, and only finds commonly
 # understood situations for x86/x86_64.
 #
-# Copyright 2009, Canonical, Ltd.
+# Copyright 2009-2010  Canonical, Ltd.
 # Author: Kees Cook <kees@ubuntu.com>
-# License: GPLv3
+#
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
+
 import sys, re
 
 class ParseSegv(object):
     def __init__(self, registers, disassembly, maps, debug=False):
         self.debug = debug
+
         self.regs = self.parse_regs(registers)
+        self.sp = None
+        for reg in ['rsp','esp']:
+            if reg in self.regs:
+                self.sp = self.regs[reg]
+
         self.line, self.pc, self.insn, self.src, self.dest = \
             self.parse_disassembly(disassembly)
+
         self.maps = self.parse_maps(maps)
 
     def find_vma(self, addr):
@@ -59,6 +72,9 @@ class ParseSegv(object):
         if len(lines)<1:
             raise ValueError, 'Failed to load empty disassembly'
         line = lines[0].strip()
+        # Drop GDB 7.1's leading $pc mark
+        if line.startswith('=>'):
+            line = line[2:].strip()
         if self.debug:
             print >>sys.stderr, line
         pc_str = line.split()[0]
@@ -66,7 +82,7 @@ class ParseSegv(object):
             pc = int(pc_str.split(':')[0],16)
         else:
             # Could not identify this instruction line
-            raise ValueError, 'Could not parse disassembly line: %s' % (pc_str)
+            raise ValueError, 'Could not parse PC "%s" from disassembly line: %s' % (pc_str, line)
         if self.debug:
             print >>sys.stderr, 'pc: 0x%08x' % (pc)
 
@@ -114,8 +130,8 @@ class ParseSegv(object):
                 if self.debug:
                     print >>sys.stderr, 'dest: %s' % (dest)
 
-        # Set up possible implicit memory destination
-        if insn in ['push','pop','pushl','popl']:
+        # Set up possible implicit memory destinations (stack actions)
+        if insn in ['push','pop','pushl','popl','call','callq','ret','retq']:
             for reg in ['rsp','esp']:
                 if reg in self.regs:
                     dest = '(%%%s)' % (reg)
@@ -287,9 +303,22 @@ class ParseSegv(object):
             details.append('disallowed I/O port operation on port %d' % (self.register_value(self.src)))
             understood = True
 
+        # Guess at falling off the stack?
+        if not understood and self.sp:
+            valid, out, short = self.validate_vma('r', self.sp, 'SP')
+            details.append(out)
+            if not valid:
+                reason.append(short)
+                understood = True
+
         if not understood:
-            reason.append('Reason could not be automatically determined.')
-            details.append('Reason could not be automatically determined.')
+            vma = self.find_vma(self.pc)
+            if vma and (vma['name'] == '[vdso]' or vma['name'] == '[vsyscall]'):
+                reason.append('Reason could not be automatically determined. (Unhandled exception in kernel code?)')
+                details.append('Reason could not be automatically determined. (Unhandled exception in kernel code?)')
+            else:
+                reason.append('Reason could not be automatically determined.')
+                details.append('Reason could not be automatically determined.')
         return understood, '\n'.join(reason), '\n'.join(details)
 
 
@@ -365,7 +394,7 @@ rdx            0xffffffffff600180   -10485376
 rsi            0x0  0
 rdi            0x7fffffffe3b0   140737488348080
 rbp            0x0  0x0
-rsp            0x7fffffffe388   0x7fffffffe388
+rsp            0x0000bfc6af24   0x0000bfc6af24
 r8             0x0  0
 r9             0x0  0
 r10            0x7fffffffe140   140737488347456
@@ -453,7 +482,7 @@ bfc57000-bfc6c000 rw-p 00000000 00:00 0          [stack]
 0x0808357d <main+61>:   ret    
 '''
 
-        class _TestParseSegv(unittest.TestCase):
+        class _T(unittest.TestCase):
             '''Test Segfault Parser'''
 
             def setUp(self):
@@ -582,6 +611,15 @@ bfc57000-bfc6c000 rw-p 00000000 00:00 0          [stack]
                 self.assertEquals(segv.src, '%ecx', segv.src)
                 self.assertEquals(segv.dest, '(%esp)', segv.dest)
 
+                # GDB 7.1
+                regs = 'esp 0x444'
+                disasm = '=> 0x08083560 <main+0>:    push %ecx\n'
+                segv = ParseSegv(regs, disasm, '')
+                self.assertEquals(segv.pc, 0x08083560, segv.pc)
+                self.assertEquals(segv.insn, 'push', segv.insn)
+                self.assertEquals(segv.src, '%ecx', segv.src)
+                self.assertEquals(segv.dest, '(%esp)', segv.dest)
+
             def test_ioport_operation(self):
                 '''I/O port violations'''
 
@@ -820,4 +858,50 @@ bfc57000-bfc6c000 rw-p 00000000 00:00 0          [stack]
                 segv = ParseSegv(regs, disasm, maps)
                 self.assertRaises(ValueError, segv.report)
 
+            def test_segv_stack_failure(self):
+                '''Handles walking off the stack'''
+
+                # Triggered via "push"
+                reg = regs + 'esp            0xbfc56ff0   0xbfc56ff0'
+                disasm = '0x08083547 <main+7>:    push  %eax'
+                segv = ParseSegv(reg, disasm, maps)
+                understood, reason, details = segv.report()
+                self.assertTrue(understood, details)
+                self.assertTrue('destination "(%esp)" (0xbfc56ff0) not located in a known VMA region (needed writable region)!' in details, details)
+
+                # Triggered via "call"
+                reg = regs + 'esp            0xbfc56fff   0xbfc56fff'
+                disasm = '0x08083547 <main+7>:    callq  0x08083540'
+                segv = ParseSegv(reg, disasm, maps)
+                understood, reason, details = segv.report()
+                self.assertTrue(understood, details)
+                self.assertTrue('destination "(%esp)" (0xbfc56fff) not located in a known VMA region (needed writable region)!' in details, details)
+
+                # Triggered via unknown reason
+                reg = regs + 'esp            0xbfc56000   0xbfc56000'
+                disasm = '''0x08083540 <main+0>:    mov    $1,%rcx'''
+                segv = ParseSegv(reg, disasm, maps)
+                understood, reason, details = segv.report()
+                self.assertTrue(understood, details)
+                self.assertTrue('SP (0xbfc56000) not located in a known VMA region (needed readable region)!' in details, details)
+
+            def test_segv_stack_failure(self):
+                '''Handles unknown segfaults in kernel'''
+
+                # Crash in valid code path
+                disasm = '''0x0056e010: ret'''
+                segv = ParseSegv(regs, disasm, maps)
+                understood, reason, details = segv.report()
+                self.assertFalse(understood, details)
+                self.assertTrue('Reason could not be automatically determined.' in details, details)
+                self.assertFalse('(Unhandled exception in kernel code?)' in details, details)
+
+                # Crash from kernel code path
+                disasm = '''0x00b67422 <__kernel_vsyscall+2>: ret'''
+                segv = ParseSegv(regs, disasm, maps)
+                understood, reason, details = segv.report()
+                self.assertFalse(understood, details)
+                self.assertTrue('Reason could not be automatically determined. (Unhandled exception in kernel code?)' in details, details)
+
+
     unittest.main()
