@@ -1,16 +1,15 @@
 # vim: set encoding=UTF-8 fileencoding=UTF-8 :
 
-'''Store, load, and handle problem reports.
+'''Store, load, and handle problem reports.'''
 
-Copyright (C) 2006 Canonical Ltd.
-Author: Martin Pitt <martin.pitt@ubuntu.com>
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
-option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
-the full text of the license.
-'''
+# Copyright (C) 2006 - 2009 Canonical Ltd.
+# Author: Martin Pitt <martin.pitt@ubuntu.com>
+# 
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
+# the full text of the license.
 
 import zlib, base64, time, UserDict, sys, gzip, struct
 from cStringIO import StringIO
@@ -198,7 +197,8 @@ class ProblemReport(UserDict.IterableUserDict):
         which will be read and its content will become the value of this key.
         'encode' specifies whether the contents will be
         gzip compressed and base64-encoded (this defaults to True). If limit is
-        set to a positive integer, the entire key will be removed. If
+        set to a positive integer, the file is not attached if it's larger
+        than the given limit, and the entire key will be removed. If
         fail_on_empty is True, reading zero bytes will cause an IOError.
 
         Files are written in RFC822 format.
@@ -359,7 +359,7 @@ class ProblemReport(UserDict.IterableUserDict):
             os.chmod(reportfile, st.st_mode)
 
     def write_mime(self, file, attach_treshold = 5, extra_headers={},
-        skip_keys=None):
+        skip_keys=None, priority_fields=None):
         '''Write MIME/Multipart RFC 2822 formatted data into file.
 
         file must be a file-like object, not a path.
@@ -379,6 +379,9 @@ class ProblemReport(UserDict.IterableUserDict):
 
         skip_keys is a set/list specifying keys which are filtered out and not
         written to the destination file.
+
+        priority_fields is a set/list specifying the order in which keys should
+        appear in the destination file.
         '''
         keys = self.data.keys()
         keys.sort()
@@ -390,6 +393,14 @@ class ProblemReport(UserDict.IterableUserDict):
             keys.remove('ProblemType')
             keys.insert(0, 'ProblemType')
 
+        if priority_fields:
+            counter = 0
+            for priority_field in priority_fields:
+                if priority_field in keys:
+                    keys.remove(priority_field)
+                    keys.insert(counter, priority_field)
+                    counter += 1
+
         for k in keys:
             if skip_keys and k in skip_keys:
                 continue
@@ -442,9 +453,13 @@ class ProblemReport(UserDict.IterableUserDict):
                 attachments.append(att)
             else:
                 # plain text value
-                if type(v) == type(u''):
-                    # convert unicode to UTF-8 str
-                    v = v.encode('UTF-8')
+
+                # ensure that byte arrays are valid UTF-8
+                if type(v) == type(''):
+                    v = v.decode('UTF-8', 'replace')
+                # convert unicode to UTF-8 str
+                assert type(v) == type(u'')
+                v = v.encode('UTF-8')
 
                 lines = len(v.splitlines())
                 if lines == 1:
@@ -521,7 +536,7 @@ class ProblemReport(UserDict.IterableUserDict):
 
 import unittest, tempfile, os, email
 
-class _ProblemReportTest(unittest.TestCase):
+class _T(unittest.TestCase):
     def test_basic_operations(self):
         '''basic creation and operation.'''
 
@@ -1379,6 +1394,46 @@ GoodText: Hi
         # no more parts
         self.assertRaises(StopIteration, msg_iter.next)
 
+    def test_write_mime_order(self):
+        '''write_mime() with keys ordered.'''
+
+        bin_value = 'AB' * 10 + '\0' * 10 + 'Z'
+
+        pr = ProblemReport(date = 'now!')
+        pr['SecondText'] = 'What'
+        pr['FirstText'] = 'Who'
+        pr['FourthText'] = 'Today'
+        pr['ThirdText'] = "I Don't Know"
+        io = StringIO()
+        pr.write_mime(io, priority_fields=['FirstText', 'SecondText',
+            'ThirdText', 'Unknown', 'FourthText'])
+        io.seek(0)
+
+        msg = email.message_from_file(io)
+        msg_iter = msg.walk()
+
+        # first part is the multipart container
+        part = msg_iter.next()
+        self.assert_(part.is_multipart())
+
+        # second part should be an inline text/plain attachments with all short
+        # fields
+        part = msg_iter.next()
+        self.assert_(not part.is_multipart())
+        self.assertEqual(part.get_content_type(), 'text/plain')
+        self.assertEqual(part.get_content_charset(), 'utf-8')
+        self.assertEqual(part.get_filename(), None)
+        self.assertEqual(part.get_payload(decode=True), '''FirstText: Who
+SecondText: What
+ThirdText: I Don't Know
+FourthText: Today
+ProblemType: Crash
+Date: now!
+''')
+
+        # no more parts
+        self.assertRaises(StopIteration, msg_iter.next)
+
     def test_updating(self):
         '''new_keys() and write() with only_new=True.'''
 
